@@ -19,9 +19,11 @@ import sys, os, re
 # General configuration
 # ---------------------
 
+language = 'en'
+
 # Add any Sphinx extension module names here, as strings. They can be extensions
 # coming with Sphinx (named 'sphinx.addons.*') or your custom ones.
-extensions = ['sphinx.ext.autodoc', 'sphinx.ext.doctest']
+extensions = ['sphinx.ext.autodoc', 'sphinx.ext.doctest', 'sphinx.ext.todo']
 
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ['_templates']
@@ -49,7 +51,7 @@ release = version
 # non-false value, then it is used:
 #today = ''
 # Else, today_fmt is used as the format for a strftime call.
-today_fmt = '%B %d, %Y'
+#today_fmt = '%B %d, %Y'
 
 # List of documents that shouldn't be included in the build.
 #unused_docs = []
@@ -126,12 +128,20 @@ latex_logo = '_static/sphinx.png'
 #latex_use_parts = True
 
 # Additional stuff for the LaTeX preamble.
-#latex_preamble = ''
+latex_elements = {
+    'fontpkg': '\\usepackage{palatino}'
+}
 
 # Documents to append as an appendix to all manuals.
 #latex_appendices = []
 
 
+# Extension options
+# -----------------
+
+todo_include_todos = True
+
+
 # Extension interface
 # -------------------
 
