@@ -6,13 +6,14 @@
     Mimic doctest by automatically executing code snippets and checking
     their results.
 
-    :copyright: 2008 by Georg Brandl.
-    :license: BSD.
+    :copyright: Copyright 2007-2009 by the Sphinx team, see AUTHORS.
+    :license: BSD, see LICENSE for details.
 """
 
 import re
 import sys
 import time
+import codecs
 import StringIO
 from os import path
 # circumvent relative import
@@ -169,7 +170,8 @@ class DocTestBuilder(Builder):
 
         date = time.strftime('%Y-%m-%d %H:%M:%S')
 
-        self.outfile = file(path.join(self.outdir, 'output.txt'), 'w')
+        self.outfile = codecs.open(path.join(self.outdir, 'output.txt'),
+                                   'w', encoding='utf-8')
         self.outfile.write('''\
 Results of doctest builder run on %s
 ==================================%s
@@ -179,11 +181,17 @@ Results of doctest builder run on %s
         self.info(text, nonl=True)
         self.outfile.write(text)
 
+    def _warn_out(self, text):
+        self.info(text, nonl=True)
+        if self.app.quiet:
+            self.warn(text)
+        self.outfile.write(text)
+
     def get_target_uri(self, docname, typ=None):
         return ''
 
     def get_outdated_docs(self):
-        return self.env.all_docs
+        return self.env.found_docs
 
     def finish(self):
         # write executive summary
@@ -200,11 +208,14 @@ Doctest summary
        self.setup_failures, s(self.setup_failures)))
         self.outfile.close()
 
+        if self.total_failures or self.setup_failures:
+            self.app.statuscode = 1
+
         sys.path[0:0] = self.config.doctest_path
 
     def write(self, build_docnames, updated_docnames, method='update'):
         if build_docnames is None:
-            build_docnames = self.env.all_docs
+            build_docnames = sorted(self.env.all_docs)
 
         self.info(bold('running tests...'))
         for docname in build_docnames:
@@ -229,8 +240,12 @@ Doctest summary
                 return isinstance(node, (nodes.literal_block, nodes.comment)) \
                         and node.has_key('testnodetype')
         for node in doctree.traverse(condition):
-            code = TestCode(node.has_key('test') and node['test'] or node.astext(),
-                            type=node.get('testnodetype', 'doctest'),
+            source = node.has_key('test') and node['test'] or node.astext()
+            if not source:
+                self.warn('no code/output in %s block at %s:%s' %
+                          (node.get('testnodetype', 'doctest'),
+                           self.env.doc2path(docname), node.line))
+            code = TestCode(source, type=node.get('testnodetype', 'doctest'),
                             lineno=node.line, options=node.get('options'))
             node_groups = node.get('groups', ['default'])
             if '*' in node_groups:
@@ -274,7 +289,7 @@ Doctest summary
             setup_doctest.globs = ns
             old_f = self.setup_runner.failures
             self.type = 'exec' # the snippet may contain multiple statements
-            self.setup_runner.run(setup_doctest, out=self._out,
+            self.setup_runner.run(setup_doctest, out=self._warn_out,
                                   clear_globs=False)
             if self.setup_runner.failures > old_f:
                 # don't run the group
@@ -284,8 +299,6 @@ Doctest summary
                 test = parser.get_doctest(code[0].code, {},
                                           group.name, filename, code[0].lineno)
                 if not test.examples:
-                    self._out('WARNING: no examples in doctest block at '
-                              + filename + ', line %s' % code[0].lineno)
                     continue
                 for example in test.examples:
                     # apply directive's comparison options
@@ -307,7 +320,7 @@ Doctest summary
             # DocTest.__init__ copies the globs namespace, which we don't want
             test.globs = ns
             # also don't clear the globs namespace after running the doctest
-            self.test_runner.run(test, out=self._out, clear_globs=False)
+            self.test_runner.run(test, out=self._warn_out, clear_globs=False)
 
 
 def setup(app):
