@@ -5,8 +5,8 @@
 
     Handlers for additional ReST roles.
 
-    :copyright: 2007-2008 by Georg Brandl.
-    :license: BSD.
+    :copyright: Copyright 2007-2009 by the Sphinx team, see AUTHORS.
+    :license: BSD, see LICENSE for details.
 """
 
 import re
@@ -15,9 +15,8 @@ from docutils import nodes, utils
 from docutils.parsers.rst import roles
 
 from sphinx import addnodes
+from sphinx.util import ws_re, caption_ref_re
 
-ws_re = re.compile(r'\s+')
-caption_ref_re = re.compile(r'^([^<]+?)\s*<(.+)>$')
 
 generic_docroles = {
     'command' : nodes.strong,
@@ -34,13 +33,16 @@ generic_docroles = {
 }
 
 for rolename, nodeclass in generic_docroles.iteritems():
-    roles.register_generic_role(rolename, nodeclass)
+    role = roles.GenericRole(rolename, nodeclass)
+    roles.register_local_role(rolename, role)
 
 
 def indexmarkup_role(typ, rawtext, etext, lineno, inliner, options={}, content=[]):
     env = inliner.document.settings.env
     if not typ:
         typ = env.config.default_role
+    else:
+        typ = typ.lower()
     text = utils.unescape(etext)
     targetid = 'index-%s' % env.index_num
     env.index_num += 1
@@ -48,19 +50,15 @@ def indexmarkup_role(typ, rawtext, etext, lineno, inliner, options={}, content=[
     targetnode = nodes.target('', '', ids=[targetid])
     inliner.document.note_explicit_target(targetnode)
     if typ == 'envvar':
-        env.note_index_entry('single', text, targetid, text)
-        env.note_index_entry('single', 'environment variable; %s' % text,
-                             targetid, text)
         indexnode['entries'] = [('single', text, targetid, text),
-                                ('single', 'environment variable; %s' % text,
+                                ('single', _('environment variable; %s') % text,
                                  targetid, text)]
         xref_nodes = xfileref_role(typ, rawtext, etext, lineno, inliner,
                                    options, content)[0]
         return [indexnode, targetnode] + xref_nodes, []
     elif typ == 'pep':
-        env.note_index_entry('single', 'Python Enhancement Proposals!PEP %s' % text,
-                             targetid, 'PEP %s' % text)
-        indexnode['entries'] = [('single', 'Python Enhancement Proposals!PEP %s' % text,
+        indexnode['entries'] = [('single',
+                                 _('Python Enhancement Proposals!PEP %s') % text,
                                  targetid, 'PEP %s' % text)]
         try:
             pepnum = int(text)
@@ -74,8 +72,6 @@ def indexmarkup_role(typ, rawtext, etext, lineno, inliner, options={}, content=[
         rn += sn
         return [indexnode, targetnode, rn], []
     elif typ == 'rfc':
-        env.note_index_entry('single', 'RFC; RFC %s' % text,
-                             targetid, 'RFC %s' % text)
         indexnode['entries'] = [('single', 'RFC; RFC %s' % text,
                                  targetid, 'RFC %s' % text)]
         try:
@@ -90,7 +86,7 @@ def indexmarkup_role(typ, rawtext, etext, lineno, inliner, options={}, content=[
         rn += sn
         return [indexnode, targetnode, rn], []
 
-roles.register_canonical_role('envvar', indexmarkup_role)
+roles.register_local_role('envvar', indexmarkup_role)
 roles.register_local_role('pep', indexmarkup_role)
 roles.register_local_role('rfc', indexmarkup_role)
 
@@ -118,6 +114,8 @@ def xfileref_role(typ, rawtext, text, lineno, inliner, options={}, content=[]):
     env = inliner.document.settings.env
     if not typ:
         typ = env.config.default_role
+    else:
+        typ = typ.lower()
     text = utils.unescape(text)
     # if the first character is a bang, don't cross-reference at all
     if text[0:1] == '!':
@@ -145,7 +143,7 @@ def xfileref_role(typ, rawtext, text, lineno, inliner, options={}, content=[]):
             # fallback: everything after '<' is the target
             target = text[brace+1:]
             title = text[:brace]
-    # special target  for Python object cross-references
+    # special target for Python object cross-references
     if typ in ('data', 'exc', 'func', 'class', 'const', 'attr', 'meth', 'mod', 'obj'):
         # fix-up parentheses in link title
         if titleistarget:
@@ -168,9 +166,17 @@ def xfileref_role(typ, rawtext, text, lineno, inliner, options={}, content=[]):
             target = target[1:]
             pnode['refspecific'] = True
     # some other special cases for the target
-    elif typ == 'option' and target[0] in '-/':
-        # strip option marker from target
-        target = target[1:]
+    elif typ == 'option':
+        program = env.currprogram
+        if titleistarget:
+            if ' ' in title and not (title.startswith('/') or title.startswith('-')):
+                program, target = re.split(' (?=-|--|/)', title, 1)
+                program = ws_re.sub('-', program)
+                target = target.strip()
+        elif ' ' in target:
+            program, target = re.split(' (?=-|--|/)', target, 1)
+            program = ws_re.sub('-', program)
+        pnode['refprogram'] = program
     elif typ == 'term':
         # normalize whitespace in definition terms (if the term reference is
         # broken over a line, a newline will be in target)
@@ -195,17 +201,17 @@ _litvar_re = re.compile('{([^}]+)}')
 
 def emph_literal_role(typ, rawtext, text, lineno, inliner, options={}, content=[]):
     text = utils.unescape(text)
-    retnodes = []
     pos = 0
+    retnode = nodes.literal(role=typ.lower())
     for m in _litvar_re.finditer(text):
         if m.start() > pos:
             txt = text[pos:m.start()]
-            retnodes.append(nodes.literal(txt, txt))
-        retnodes.append(nodes.emphasis('', '', nodes.literal(m.group(1), m.group(1))))
+            retnode += nodes.Text(txt, txt)
+        retnode += nodes.emphasis(m.group(1), m.group(1))
         pos = m.end()
     if pos < len(text):
-        retnodes.append(nodes.literal(text[pos:], text[pos:]))
-    return retnodes, []
+        retnode += nodes.Text(text[pos:], text[pos:])
+    return [retnode], []
 
 
 specific_docroles = {
@@ -218,22 +224,23 @@ specific_docroles = {
     'meth': xfileref_role,
     'obj': xfileref_role,
     'cfunc' : xfileref_role,
-    'cdata' : xfileref_role,
-    'ctype' : xfileref_role,
-    'cmacro' : xfileref_role,
+    'cmember': xfileref_role,
+    'cdata': xfileref_role,
+    'ctype': xfileref_role,
+    'cmacro': xfileref_role,
 
-    'mod' : xfileref_role,
+    'mod': xfileref_role,
 
     'keyword': xfileref_role,
     'ref': xfileref_role,
-    'token' : xfileref_role,
+    'token': xfileref_role,
     'term': xfileref_role,
     'option': xfileref_role,
 
-    'menuselection' : menusel_role,
-    'file' : emph_literal_role,
-    'samp' : emph_literal_role,
+    'menuselection': menusel_role,
+    'file': emph_literal_role,
+    'samp': emph_literal_role,
 }
 
 for rolename, func in specific_docroles.iteritems():
-    roles.register_canonical_role(rolename, func)
+    roles.register_local_role(rolename, func)
