@@ -5,8 +5,8 @@
 
     Test the entire build process with the test root.
 
-    :copyright: 2008 by Georg Brandl.
-    :license: BSD.
+    :copyright: Copyright 2007-2009 by the Sphinx team, see AUTHORS.
+    :license: BSD, see LICENSE for details.
 """
 
 import os
@@ -77,7 +77,8 @@ HTML_XPATH = {
     },
     'contents.html': {
         ".//meta[@name='hc'][@content='hcval']": '',
-        ".//td[@class='label']": '[Ref1]',
+        #".//td[@class='label']": '[Ref1]',   # docutils 0.5 only
+        ".//td[@class='label']": '',
         ".//li[@class='toctree-l1']/a": 'Testing various markup',
         ".//li[@class='toctree-l2']/a": 'Admonitions',
         ".//title": 'Sphinx <Tests>',
@@ -100,7 +101,7 @@ class NslessParser(ET.XMLParser):
             return name
 
 
-@with_app(buildername='html', warning=html_warnfile)
+@with_app(buildername='html', warning=html_warnfile, cleanenv=True)
 def test_html(app):
     app.builder.build_all()
     html_warnings = html_warnfile.getvalue().replace(os.sep, '/')
@@ -127,7 +128,7 @@ def test_html(app):
                                'path %s in %s' % (text, path, fname))
 
 
-@with_app(buildername='latex', warning=latex_warnfile)
+@with_app(buildername='latex', warning=latex_warnfile, cleanenv=True)
 def test_latex(app):
     LaTeXTranslator.ignore_missing_images = True
     app.builder.build_all()
@@ -183,11 +184,11 @@ def test_latex(app):
 
 # just let the remaining ones run for now
 
-@with_app(buildername='linkcheck')
+@with_app(buildername='linkcheck', cleanenv=True)
 def test_linkcheck(app):
     app.builder.build_all()
 
-@with_app(buildername='text')
+@with_app(buildername='text', cleanenv=True)
 def test_text(app):
     app.builder.build_all()
 
